@@ -55,6 +55,8 @@
 #include "burn-task.h"
 #include "brasero-caps-burn.h"
 
+#include "brasero-drive-priv.h"
+
 #include "brasero-volume.h"
 #include "brasero-drive.h"
 
@@ -145,7 +147,6 @@ static guint brasero_burn_signals [LAST_SIGNAL] = { 0 };
 
 #define MAX_EJECT_ATTEMPTS	5
 #define MAX_MOUNT_ATTEMPTS	40
-#define MAX_REPROBE_ATTEMPTS	80
 
 #define MOUNT_TIMEOUT		500
 
@@ -259,8 +260,6 @@ brasero_burn_sleep (BraseroBurn *burn, gint msec)
 static BraseroBurnResult
 brasero_burn_reprobe (BraseroBurn *burn)
 {
-	guint attempts = 0;
-	BraseroMedium *medium;
 	BraseroBurnPrivate *priv;
 	BraseroBurnResult result = BRASERO_BURN_OK;
 
@@ -270,14 +269,55 @@ brasero_burn_reprobe (BraseroBurn *burn)
 
 	/* reprobe the medium and wait for it to be probed */
 	brasero_drive_reprobe (priv->dest);
-	while (attempts < MAX_REPROBE_ATTEMPTS && (medium = brasero_drive_get_medium (priv->dest)) == NULL) {
+	while (brasero_drive_probing (priv->dest)) {
 		result = brasero_burn_sleep (burn, 250);
-		attempts ++;
+		if (result != BRASERO_BURN_OK)
+			return result;
 	}
 
 	return result;
 }
 
+static BraseroBurnResult
+brasero_burn_unmount (BraseroBurn *self,
+                      BraseroMedium *medium,
+                      GError **error)
+{
+	guint counter = 0;
+
+	if (!medium)
+		return BRASERO_BURN_OK;
+
+	/* Retry several times, since sometimes the drives are really busy */
+	while (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
+		BraseroBurnResult result;
+
+		counter ++;
+		if (counter > MAX_EJECT_ATTEMPTS) {
+			BRASERO_BURN_LOG ("Max attempts reached at unmounting");
+			if (error && !(*error))
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_DRIVE_BUSY,
+					     "%s. %s",
+					     _("The drive is busy"),
+					     _("Make sure another application is not using it"));
+			return BRASERO_BURN_ERR;
+		}
+
+		BRASERO_BURN_LOG ("Retrying unmounting");
+		result = brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL);
+		if (result != BRASERO_BURN_OK)
+			return result;
+
+		result = brasero_burn_sleep (self, 500);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
+	return BRASERO_BURN_OK;
+}
+
 static BraseroBurnResult
 brasero_burn_eject (BraseroBurn *self,
 		    BraseroDrive *drive,
@@ -291,7 +331,18 @@ brasero_burn_eject (BraseroBurn *self,
 	brasero_burn_sleep (self, 500);
 
 	/* Retry several times, since sometimes the drives are really busy */
-	while (brasero_drive_get_medium (drive)) {
+	while (brasero_drive_get_medium (drive) || brasero_drive_probing (drive)) {
+		GError *ret_error;
+		BraseroBurnResult result;
+
+		while (brasero_drive_probing (drive)) {
+			result = brasero_burn_sleep (self, 500);
+			if (result != BRASERO_BURN_OK)
+				return result;
+
+			continue;
+		}
+
 		counter ++;
 		if (counter > MAX_EJECT_ATTEMPTS) {
 			gchar *name;
@@ -312,8 +363,18 @@ brasero_burn_eject (BraseroBurn *self,
 		}
 
 		BRASERO_BURN_LOG ("Retrying ejection");
-		brasero_drive_eject (drive, TRUE, NULL);
-		brasero_burn_sleep (self, 500);
+
+		ret_error = NULL;
+		brasero_drive_eject (drive, TRUE, &ret_error);
+
+		if (ret_error) {
+			BRASERO_BURN_LOG ("Ejection error: %s", ret_error->message);
+			g_error_free (ret_error);
+		}
+
+		result = brasero_burn_sleep (self, 500);
+		if (result != BRASERO_BURN_OK)
+			return result;
 	}
 
 	return BRASERO_BURN_OK;
@@ -489,13 +550,12 @@ brasero_burn_ask_for_src_media (BraseroBurn *burn,
 				BraseroMedia required_media,
 				GError **error)
 {
-	BraseroMedia media;
 	BraseroMedium *medium;
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 
 	medium = brasero_drive_get_medium (priv->src);
-	media = brasero_medium_get_status (medium);
-	if (media != BRASERO_MEDIUM_NONE) {
+	if (brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE
+	||  brasero_drive_probing (priv->src)) {
 		BraseroBurnResult result;
 		result = brasero_burn_eject_src_media (burn, error);
 		if (result != BRASERO_BURN_OK)
@@ -530,7 +590,8 @@ brasero_burn_ask_for_dest_media (BraseroBurn *burn,
 	}
 
 	medium = brasero_drive_get_medium (priv->dest);
-	if (medium || brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE) {
+	if (brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE
+	||  brasero_drive_probing (priv->dest)) {
 		BraseroBurnResult result;
 
 		result = brasero_burn_eject_dest_media (burn, error);
@@ -568,6 +629,12 @@ brasero_burn_lock_src_media (BraseroBurn *burn,
 
 again:
 
+	while (brasero_drive_probing (priv->src)) {
+		result = brasero_burn_sleep (burn, 500);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
 	medium = brasero_drive_get_medium (priv->src);
 	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))) {
 		if (!brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL))
@@ -655,6 +722,12 @@ brasero_burn_lock_rewritable_media (BraseroBurn *burn,
 
  again:
 
+	while (brasero_drive_probing (priv->dest)) {
+		result = brasero_burn_sleep (burn, 500);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
 	medium = brasero_drive_get_medium (priv->dest);
 	if (!brasero_medium_can_be_rewritten (medium)) {
 		g_set_error (error,
@@ -801,6 +874,12 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 		return BRASERO_BURN_ERR;
 	}
 
+	while (brasero_drive_probing (priv->dest)) {
+		result = brasero_burn_sleep (burn, 500);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
 	medium = brasero_drive_get_medium (priv->dest);
 	if (!medium) {
 		result = BRASERO_BURN_NEED_RELOAD;
@@ -872,6 +951,29 @@ brasero_burn_lock_dest_media (BraseroBurn *burn,
 	brasero_burn_session_get_input_type (priv->session, input);
 	flags = brasero_burn_session_get_flags (priv->session);
 
+	if (brasero_track_type_get_has_image (input)) {
+		goffset medium_sec = 0;
+		goffset session_sec = 0;
+
+		/* This test is only valid when it's an image
+		 * as input which comes handy when copying
+		 * from the same drive as we're sure we do 
+		 * have an image. */
+		brasero_medium_get_capacity (medium,
+		                             NULL,
+		                             &medium_sec);
+		brasero_burn_session_get_size (priv->session,
+		                               &session_sec,
+		                               NULL);
+
+		if (session_sec > medium_sec) {
+			BRASERO_BURN_LOG ("Not enough space for image %"G_GOFFSET_FORMAT"/%"G_GOFFSET_FORMAT, session_sec, medium_sec);
+			berror = BRASERO_BURN_ERROR_MEDIUM_SPACE;
+			result = BRASERO_BURN_NEED_RELOAD;
+			goto end;
+		}
+	}
+
 	if (must_blank) {
 		/* There is an error if APPEND was set since this disc is not
 		 * supported without a prior blanking. */
@@ -1020,6 +1122,12 @@ brasero_burn_lock_checksum_media (BraseroBurn *burn,
 
 again:
 
+	while (brasero_drive_probing (priv->dest)) {
+		result = brasero_burn_sleep (burn, 500);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
 	medium = brasero_drive_get_medium (priv->dest);
 	media = brasero_medium_get_status (medium);
 	error_type = BRASERO_BURN_ERROR_NONE;
@@ -1151,13 +1259,10 @@ brasero_burn_progress_changed (BraseroTaskCtx *task,
 			       BraseroBurn *burn)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
-	BraseroBurnAction action = BRASERO_BURN_ACTION_NONE;
 	gdouble overall_progress = -1.0;
 	gdouble task_progress = -1.0;
 	glong time_remaining = -1;
 
-	brasero_task_ctx_get_current_action (task, &action);
-
 	/* get the task current progress */
 	if (brasero_task_ctx_get_progress (task, &task_progress) == BRASERO_BURN_OK) {
 		brasero_task_ctx_get_remaining_time (task, &time_remaining);
@@ -1347,23 +1452,30 @@ brasero_burn_run_eraser (BraseroBurn *burn, GError **error)
 	BraseroDrive *drive;
 	BraseroMedium *medium;
 	BraseroBurnPrivate *priv;
+	BraseroBurnResult result;
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
 	drive = brasero_burn_session_get_burner (priv->session);
 	medium = brasero_drive_get_medium (drive);
-	if (brasero_volume_is_mounted (BRASERO_VOLUME (medium))
-	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_DRIVE_BUSY,
-			     "%s. %s",
-			     _("The drive is busy"),
-			     _("Make sure another application is not using it"));
-		return BRASERO_BURN_ERR;
-	}
 
-	return brasero_task_run (priv->task, error);
+	result = brasero_burn_unmount (burn, medium, error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	result = brasero_task_run (priv->task, error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
+	/* Reprobe. It can happen (like with dvd+rw-format) that
+	 * for the whole OS, the disc doesn't exist during the 
+	 * formatting. Wait for the disc to reappear */
+	/*  Likewise, this is necessary when we do a
+	 * simulation before blanking since it blanked the disc
+	 * and then to create all tasks necessary for the real
+	 * burning operation, we'll need the real medium status 
+	 * not to include a blanking job again. */
+	return brasero_burn_reprobe (burn);
 }
 
 static BraseroBurnResult
@@ -1383,18 +1495,11 @@ brasero_burn_run_imager (BraseroBurn *burn,
 start:
 
 	medium = brasero_drive_get_medium (src);
-
-	/* This is just in case */
-	if (medium
-	&&  brasero_volume_is_mounted (BRASERO_VOLUME (medium))
-	&& !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_DRIVE_BUSY,
-			     "%s. %s",
-			     _("The drive is busy"),
-			     _("Make sure another application is not using it"));
-		return BRASERO_BURN_ERR;
+	if (medium) {
+		/* This is just in case */
+		result = brasero_burn_unmount (burn, medium, error);
+		if (result != BRASERO_BURN_OK)
+			return result;
 	}
 
 	/* If it succeeds then the new track(s) will be at the top of
@@ -1524,7 +1629,7 @@ brasero_burn_can_use_drive_exclusively (BraseroBurn *burn,
 	while (!brasero_drive_can_use_exclusively (drive)) {
 		BRASERO_BURN_LOG ("Device busy, retrying in 250 ms");
 		result = brasero_burn_sleep (burn, 250);
-		if (result == BRASERO_BURN_CANCEL)
+		if (result != BRASERO_BURN_OK)
 			return result;
 	}
 
@@ -1552,6 +1657,19 @@ brasero_burn_run_recorder (BraseroBurn *burn, GError **error)
 	burner = brasero_burn_session_get_burner (priv->session);
 	burnt_medium = brasero_drive_get_medium (burner);
 
+start:
+
+	/* this is just in case */
+	if (BRASERO_BURN_SESSION_NO_TMP_FILE (priv->session)) {
+		result = brasero_burn_unmount (burn, src_medium, error);
+		if (result != BRASERO_BURN_OK)
+			return result;
+	}
+
+	result = brasero_burn_unmount (burn, burnt_medium, error);
+	if (result != BRASERO_BURN_OK)
+		return result;
+
 	/* before we start let's see if that drive can be used exclusively.
 	 * Of course, it's not really safe since a process could take a lock
 	 * just after us but at least it'll give some time to HAL and friends
@@ -1562,33 +1680,6 @@ brasero_burn_run_recorder (BraseroBurn *burn, GError **error)
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-start:
-
-	/* this is just in case */
-	if (BRASERO_BURN_SESSION_NO_TMP_FILE (priv->session)
-	&&  src_medium
-	&&  brasero_volume_is_mounted (BRASERO_VOLUME (src_medium))
-	&& !brasero_volume_umount (BRASERO_VOLUME (src_medium), TRUE, NULL)) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_DRIVE_BUSY,
-			     "%s. %s",
-			     _("The drive is busy"),
-			     _("Make sure another application is not using it"));
-		return BRASERO_BURN_ERR;
-	}
-
-	if (brasero_volume_is_mounted (BRASERO_VOLUME (burnt_medium))
-	&& !brasero_volume_umount (BRASERO_VOLUME (burnt_medium), TRUE, NULL)) {
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_DRIVE_BUSY,
-			     "%s. %s",
-			     _("The drive is busy"),
-			     _("Make sure another application is not using it"));
-		return BRASERO_BURN_ERR;
-	}
-
 	/* actual running of task */
 	result = brasero_task_run (priv->task, &ret_error);
 
@@ -1660,7 +1751,10 @@ start:
 		g_error_free (ret_error);
 		ret_error = NULL;
 
-		brasero_burn_sleep (burn, 2000);
+		result = brasero_burn_sleep (burn, 2000);
+		if (result != BRASERO_BURN_OK)
+			return result;
+
 		has_slept = TRUE;
 
 		/* set speed at 8x max and even less if speed  */
@@ -1737,218 +1831,6 @@ start:
 	return BRASERO_BURN_ERR;
 }
 
-/* FIXME: at the moment we don't allow for mixed CD type */
-static BraseroBurnResult
-brasero_burn_run_tasks (BraseroBurn *burn,
-			gboolean erase_allowed,
-			GError **error)
-{
-	BraseroBurnResult result;
-	GSList *tasks, *next, *iter;
-	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
-
-	tasks = brasero_burn_caps_new_task (priv->caps,
-					    priv->session,
-					    error);
-	if (!tasks)
-		return BRASERO_BURN_NOT_SUPPORTED;
-
-	priv->tasks_done = 0;
-	priv->task_nb = g_slist_length (tasks);
-	BRASERO_BURN_LOG ("%i tasks to perform", priv->task_nb);
-
-	/* run all imaging tasks first */
-	for (iter = tasks; iter; iter = next) {
-		goffset len = 0;
-		BraseroDrive *drive;
-		BraseroMedium *medium;
-		BraseroTaskAction action;
-
-		next = iter->next;
-		priv->task = iter->data;
-		tasks = g_slist_remove (tasks, priv->task);
-
-		g_signal_connect (priv->task,
-				  "progress-changed",
-				  G_CALLBACK (brasero_burn_progress_changed),
-				  burn);
-		g_signal_connect (priv->task,
-				  "action-changed",
-				  G_CALLBACK (brasero_burn_action_changed),
-				  burn);
-
-		/* see what type of task it is. It could be a blank/erase one */
-		action = brasero_task_ctx_get_action (BRASERO_TASK_CTX (priv->task));
-		if (action == BRASERO_TASK_ACTION_ERASE) {
-			/* This is to avoid a potential problem when running a 
-			 * dummy session first. When running dummy session the 
-			 * media gets erased if need be. Since it is not
-			 * reloaded afterwards, for brasero it has still got 
-			 * data on it when we get to the real recording. */
-			if (erase_allowed) {
-				result = brasero_burn_run_eraser (burn, error);
-				if (result != BRASERO_BURN_OK)
-					break;
-			}
-			else
-				result = BRASERO_BURN_OK;
-
-			g_object_unref (priv->task);
-			priv->task = NULL;
-			priv->tasks_done ++;
-
-			/* Reprobe. It can happen (like with dvd+rw-format) that
-			 * for the whole OS, the disc doesn't exist during the 
-			 * formatting. Wait for the disc to reappear */
-			/*  Likewise, this is necessary when we do a
-			 * simulation before blanking since it blanked the disc
-			 * and then to create all tasks necessary for the real
-			 * burning operation, we'll need the real medium status 
-			 * not to include a blanking job again. */
-			result = brasero_burn_reprobe (burn);
-			if (result != BRASERO_BURN_OK)
-				break;
-
-			continue;
-		}
-
-		/* Init the task and set the task output size. The task should
-		 * then check that the disc has enough space. If the output is
-		 * to the hard drive it will be done afterwards when not in fake
-		 * mode. */
-		result = brasero_burn_run_imager (burn, TRUE, error);
-		if (result != BRASERO_BURN_OK)
-			break;
-
-		/* try to get the output size */
-		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
-							  &len,
-							  NULL);
-
-		drive = brasero_burn_session_get_burner (priv->session);
-		medium = brasero_drive_get_medium (drive);
-
-		if (brasero_burn_session_get_flags (priv->session) & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
-			priv->session_start = brasero_medium_get_next_writable_address (medium);
-		else
-			priv->session_start = 0;
-
-		priv->session_end = priv->session_start + len;
-
-		BRASERO_BURN_LOG ("Burning from %lld to %lld",
-				  priv->session_start,
-				  priv->session_end);
-
-		/* see if we reached a recording task: it's the last task */
-		if (!next) {
-			if (brasero_burn_session_is_dest_file (priv->session))
-				result = brasero_burn_run_imager (burn, FALSE, error);
-			else
-				result = brasero_burn_run_recorder (burn, error);
-
-			if (result == BRASERO_BURN_OK)
-				priv->tasks_done ++;
-
-			break;
-		}
-
-		/* run the imager */
-		result = brasero_burn_run_imager (burn, FALSE, error);
-		if (result != BRASERO_BURN_OK)
-			break;
-
-		g_object_unref (priv->task);
-		priv->task = NULL;
-		priv->tasks_done ++;
-	}
-
-	if (priv->task) {
-		g_object_unref (priv->task);
-		priv->task = NULL;
-	}
-
-	g_slist_foreach (tasks, (GFunc) g_object_unref, NULL);
-	g_slist_free (tasks);
-
-	return result;
-}
-
-static BraseroBurnResult
-brasero_burn_check_real (BraseroBurn *self,
-			 BraseroTrack *track,
-			 GError **error)
-{
-	BraseroMedium *medium;
-	BraseroBurnResult result;
-	BraseroBurnPrivate *priv;
-	BraseroChecksumType checksum_type;
-
-	priv = BRASERO_BURN_PRIVATE (self);
-
-	BRASERO_BURN_LOG ("Starting to check track integrity");
-
-	checksum_type = brasero_track_get_checksum_type (track);
-
-	/* if the input is a DISC and there isn't any checksum specified that 
-	 * means the checksum file is on the disc. */
-	medium = brasero_drive_get_medium (priv->dest);
-
-	/* get the task and run it */
-	priv->task = brasero_burn_caps_new_checksuming_task (priv->caps,
-							     priv->session,
-							     error);
-	if (priv->task) {
-		priv->task_nb = 1;
-		priv->tasks_done = 0;
-		g_signal_connect (priv->task,
-				  "progress-changed",
-				  G_CALLBACK (brasero_burn_progress_changed),
-				  self);
-		g_signal_connect (priv->task,
-				  "action-changed",
-				  G_CALLBACK (brasero_burn_action_changed),
-				  self);
-
-
-		/* make sure one last time it is not mounted IF and only IF the
-		 * checksum type is NOT FILE_MD5 */
-		/* it seems to work without unmounting ... */
-		/* if (medium
-		 * &&  brasero_volume_is_mounted (BRASERO_VOLUME (medium))
-		 * && !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
-		 *	g_set_error (error,
-		 *		     BRASERO_BURN_ERROR,
-		 *		     BRASERO_BURN_ERROR_DRIVE_BUSY,
-		 *		     "%s. %s",
-		 *		     _("The drive is busy"),
-		 *		     _("Make sure another application is not using it"));
-		 *	return BRASERO_BURN_ERR;
-		 * }
-		 */
-
-		result = brasero_task_run (priv->task, error);
-		g_signal_emit (self,
-			       brasero_burn_signals [PROGRESS_CHANGED_SIGNAL],
-			       0,
-			       1.0,
-			       1.0,
-			       -1);
-
-		if (result == BRASERO_BURN_OK || result == BRASERO_BURN_CANCEL)
-			brasero_burn_action_changed_real (self,
-							  BRASERO_BURN_ACTION_FINISHED);
-
-		g_object_unref (priv->task);
-		priv->task = NULL;
-	}
-	else {
-		BRASERO_BURN_LOG ("The track cannot be checked");
-		result = BRASERO_BURN_NOT_SUPPORTED;
-	}
-
-	return result;
-}
-
 static BraseroBurnResult
 brasero_burn_check_session_consistency (BraseroBurn *burn,
 					GError **error)
@@ -2063,24 +1945,304 @@ brasero_burn_check_session_consistency (BraseroBurn *burn,
 	return BRASERO_BURN_OK;
 }
 
+/* FIXME: at the moment we don't allow for mixed CD type */
+static BraseroBurnResult
+brasero_burn_run_tasks (BraseroBurn *burn,
+			gboolean erase_allowed,
+                        gboolean *dummy_session,
+			GError **error)
+{
+	BraseroBurnResult result;
+	GSList *tasks, *next, *iter;
+	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
+
+	/* push the session settings to keep the original session untainted */
+	brasero_burn_session_push_settings (priv->session);
+
+	/* check flags consistency */
+	result = brasero_burn_check_session_consistency (burn, error);
+	if (result != BRASERO_BURN_OK) {
+		brasero_burn_session_pop_settings (priv->session);
+		return result;
+	}
+
+	tasks = brasero_burn_caps_new_task (priv->caps,
+					    priv->session,
+					    error);
+	if (!tasks) {
+		brasero_burn_session_pop_settings (priv->session);
+		return BRASERO_BURN_NOT_SUPPORTED;
+	}
+
+	priv->tasks_done = 0;
+	priv->task_nb = g_slist_length (tasks);
+	BRASERO_BURN_LOG ("%i tasks to perform", priv->task_nb);
+
+	/* run all imaging tasks first */
+	for (iter = tasks; iter; iter = next) {
+		goffset len = 0;
+		BraseroDrive *drive;
+		BraseroMedium *medium;
+		BraseroTaskAction action;
+
+		next = iter->next;
+		priv->task = iter->data;
+		tasks = g_slist_remove (tasks, priv->task);
+
+		g_signal_connect (priv->task,
+				  "progress-changed",
+				  G_CALLBACK (brasero_burn_progress_changed),
+				  burn);
+		g_signal_connect (priv->task,
+				  "action-changed",
+				  G_CALLBACK (brasero_burn_action_changed),
+				  burn);
+
+		/* see what type of task it is. It could be a blank/erase one. */
+		/* FIXME!
+		 * If so then that's time to test the size of the image against
+		 * the size of the disc since erasing/formatting is always left
+		 * for the end, just before burning. We would not like to 
+		 * blank a disc and tell the user right after that the size of
+		 * the disc is not enough. */
+		action = brasero_task_ctx_get_action (BRASERO_TASK_CTX (priv->task));
+		if (action == BRASERO_TASK_ACTION_ERASE) {
+			BraseroTrackType *type;
+
+			/* FIXME: how could it be possible for a drive to test
+			 * with a CLOSED CDRW for example. Maybe we should
+			 * format/blank anyway. */
+
+			/* This is to avoid blanking a medium without knowing
+			 * if the data will fit on it. At this point we do know 
+			 * what the size of the data is going to be. */
+			type = brasero_track_type_new ();
+			brasero_burn_session_get_input_type (priv->session, type);
+			if (brasero_track_type_get_has_image (type)
+			||  brasero_track_type_get_has_medium (type)) {
+				BraseroMedium *medium;
+				goffset session_sec = 0;
+				goffset medium_sec = 0;
+
+				medium = brasero_drive_get_medium (priv->dest);
+				brasero_medium_get_capacity (medium,
+							     NULL,
+							     &medium_sec);
+
+				brasero_burn_session_get_size (priv->session,
+							       &session_sec,
+							       NULL);
+
+				if (session_sec > medium_sec) {
+					BRASERO_BURN_LOG ("Not enough space on medium %"G_GOFFSET_FORMAT"/%"G_GOFFSET_FORMAT, session_sec, medium_sec);
+					result = brasero_burn_reload_dest_media (burn,  BRASERO_BURN_ERROR_MEDIUM_SPACE, error);
+					if (result != BRASERO_BURN_OK)
+						break;
+				}
+			}
+			brasero_track_type_free (type);
+
+			/* This is to avoid a potential problem when running a 
+			 * dummy session first. When running dummy session the 
+			 * media gets erased if need be. Since it is not
+			 * reloaded afterwards, for brasero it has still got 
+			 * data on it when we get to the real recording. */
+			if (erase_allowed) {
+				result = brasero_burn_run_eraser (burn, error);
+				if (result != BRASERO_BURN_OK)
+					break;
+
+				/* Since we blanked/formatted we need to recheck the burn 
+				 * flags with the new medium type as some flags could have
+				 * been given the benefit of the double (MULTI with a CLOSED
+				 * CD for example). Recheck the original flags as they were
+				 * passed. */
+				/* FIXME: for some important flags we should warn the user
+				 * that it won't be possible */
+				brasero_burn_session_pop_settings (priv->session);
+				brasero_burn_session_push_settings (priv->session);
+				result = brasero_burn_check_session_consistency (burn, error);
+				if (result != BRASERO_BURN_OK)
+					break;
+			}
+			else
+				result = BRASERO_BURN_OK;
+
+			g_object_unref (priv->task);
+			priv->task = NULL;
+			priv->tasks_done ++;
+
+			continue;
+		}
+
+		/* Init the task and set the task output size. The task should
+		 * then check that the disc has enough space. If the output is
+		 * to the hard drive it will be done afterwards when not in fake
+		 * mode. */
+		result = brasero_burn_run_imager (burn, TRUE, error);
+		if (result != BRASERO_BURN_OK)
+			break;
+
+		/* try to get the output size */
+		brasero_task_ctx_get_session_output_size (BRASERO_TASK_CTX (priv->task),
+							  &len,
+							  NULL);
+
+		drive = brasero_burn_session_get_burner (priv->session);
+		medium = brasero_drive_get_medium (drive);
+
+		if (brasero_burn_session_get_flags (priv->session) & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND))
+			priv->session_start = brasero_medium_get_next_writable_address (medium);
+		else
+			priv->session_start = 0;
+
+		priv->session_end = priv->session_start + len;
+
+		BRASERO_BURN_LOG ("Burning from %lld to %lld",
+				  priv->session_start,
+				  priv->session_end);
+
+		/* see if we reached a recording task: it's the last task */
+		if (!next) {
+			if (!brasero_burn_session_is_dest_file (priv->session)) {
+				*dummy_session = (brasero_burn_session_get_flags (priv->session) & BRASERO_BURN_FLAG_DUMMY);
+				result = brasero_burn_run_recorder (burn, error);
+			}
+			else
+				result = brasero_burn_run_imager (burn, FALSE, error);
+
+			if (result == BRASERO_BURN_OK)
+				priv->tasks_done ++;
+
+			break;
+		}
+
+		/* run the imager */
+		result = brasero_burn_run_imager (burn, FALSE, error);
+		if (result != BRASERO_BURN_OK)
+			break;
+
+		g_object_unref (priv->task);
+		priv->task = NULL;
+		priv->tasks_done ++;
+	}
+
+	/* restore the session settings. Keep the used flags
+	 * nevertheless to make sure we actually use the flags that were
+	 * set after checking for session consistency. */
+	brasero_burn_session_pop_settings (priv->session);
+
+	if (priv->task) {
+		g_object_unref (priv->task);
+		priv->task = NULL;
+	}
+
+	g_slist_foreach (tasks, (GFunc) g_object_unref, NULL);
+	g_slist_free (tasks);
+
+	return result;
+}
+
+static BraseroBurnResult
+brasero_burn_check_real (BraseroBurn *self,
+			 BraseroTrack *track,
+			 GError **error)
+{
+	BraseroMedium *medium;
+	BraseroBurnResult result;
+	BraseroBurnPrivate *priv;
+	BraseroChecksumType checksum_type;
+
+	priv = BRASERO_BURN_PRIVATE (self);
+
+	BRASERO_BURN_LOG ("Starting to check track integrity");
+
+	checksum_type = brasero_track_get_checksum_type (track);
+
+	/* if the input is a DISC and there isn't any checksum specified that 
+	 * means the checksum file is on the disc. */
+	medium = brasero_drive_get_medium (priv->dest);
+
+	/* get the task and run it skip it otherwise */
+	priv->task = brasero_burn_caps_new_checksuming_task (priv->caps,
+							     priv->session,
+							     NULL);
+	if (priv->task) {
+		priv->task_nb = 1;
+		priv->tasks_done = 0;
+		g_signal_connect (priv->task,
+				  "progress-changed",
+				  G_CALLBACK (brasero_burn_progress_changed),
+				  self);
+		g_signal_connect (priv->task,
+				  "action-changed",
+				  G_CALLBACK (brasero_burn_action_changed),
+				  self);
+
+
+		/* make sure one last time it is not mounted IF and only IF the
+		 * checksum type is NOT FILE_MD5 */
+		/* it seems to work without unmounting ... */
+		/* if (medium
+		 * &&  brasero_volume_is_mounted (BRASERO_VOLUME (medium))
+		 * && !brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL)) {
+		 *	g_set_error (error,
+		 *		     BRASERO_BURN_ERROR,
+		 *		     BRASERO_BURN_ERROR_DRIVE_BUSY,
+		 *		     "%s. %s",
+		 *		     _("The drive is busy"),
+		 *		     _("Make sure another application is not using it"));
+		 *	return BRASERO_BURN_ERR;
+		 * }
+		 */
+
+		result = brasero_task_run (priv->task, error);
+		g_signal_emit (self,
+			       brasero_burn_signals [PROGRESS_CHANGED_SIGNAL],
+			       0,
+			       1.0,
+			       1.0,
+			       -1);
+
+		if (result == BRASERO_BURN_OK || result == BRASERO_BURN_CANCEL)
+			brasero_burn_action_changed_real (self,
+							  BRASERO_BURN_ACTION_FINISHED);
+
+		g_object_unref (priv->task);
+		priv->task = NULL;
+	}
+	else {
+		BRASERO_BURN_LOG ("The track cannot be checked");
+		return BRASERO_BURN_OK;
+	}
+
+	return result;
+}
+
 static void
 brasero_burn_unset_checksums (BraseroBurn *self)
 {
 	GSList *tracks;
+	BraseroTrackType *type;
 	BraseroBurnPrivate *priv;
 
 	priv = BRASERO_BURN_PRIVATE (self);
 
 	tracks = brasero_burn_session_get_tracks (priv->session);
+	type = brasero_track_type_new ();
 	for (; tracks; tracks = tracks->next) {
 		BraseroTrack *track;
 
-		/* unset checksum (might depend from copy to another). */
 		track = tracks->data;
-		brasero_track_set_checksum (track,
-					    BRASERO_CHECKSUM_NONE,
-					    NULL);
+		brasero_track_get_track_type (track, type);
+		if (!brasero_track_type_get_has_image (type)
+		&& !brasero_track_type_get_has_medium (type))
+			brasero_track_set_checksum (track,
+						    BRASERO_CHECKSUM_NONE,
+						    NULL);
 	}
+
+	brasero_track_type_free (type);
 }
 
 static BraseroBurnResult
@@ -2088,8 +2250,8 @@ brasero_burn_record_session (BraseroBurn *burn,
 			     gboolean erase_allowed,
 			     GError **error)
 {
+	gboolean dummy_session = FALSE;
 	const gchar *checksum = NULL;
-	BraseroBurnFlag session_flags;
 	BraseroTrack *track = NULL;
 	BraseroChecksumType type;
 	BraseroBurnPrivate *priv;
@@ -2100,22 +2262,11 @@ brasero_burn_record_session (BraseroBurn *burn,
 
 	priv = BRASERO_BURN_PRIVATE (burn);
 
-	/* unset checksum since no image has the exact same even if it is 
-	 * created from the same files */
+	/* unset checksum since no image has the exact
+	 * same even if it is created from the same files */
 	brasero_burn_unset_checksums (burn);
 
-	session_flags = BRASERO_BURN_FLAG_NONE;
 	do {
-		/* push the session settings to keep the original session untainted */
-		brasero_burn_session_push_settings (priv->session);
-
-		/* check flags consistency */
-		result = brasero_burn_check_session_consistency (burn, error);
-		if (result != BRASERO_BURN_OK) {
-			brasero_burn_session_pop_settings (priv->session);
-			break;
-		}
-
 		if (ret_error) {
 			g_error_free (ret_error);
 			ret_error = NULL;
@@ -2123,13 +2274,8 @@ brasero_burn_record_session (BraseroBurn *burn,
 
 		result = brasero_burn_run_tasks (burn,
 						 erase_allowed,
+		                                 &dummy_session,
 						 &ret_error);
-
-		/* restore the session settings. Keep the used flags
-		 * nevertheless to make sure we actually use the flags that were
-		 * set after checking for session consistency. */
-		session_flags = brasero_burn_session_get_flags (priv->session);
-		brasero_burn_session_pop_settings (priv->session);
 	} while (result == BRASERO_BURN_RETRY);
 
 	if (result != BRASERO_BURN_OK) {
@@ -2148,7 +2294,7 @@ brasero_burn_record_session (BraseroBurn *burn,
 	if (brasero_burn_session_is_dest_file (priv->session))
 		return BRASERO_BURN_OK;
 
-	if (session_flags & BRASERO_BURN_FLAG_DUMMY) {
+	if (dummy_session) {
 		/* if we are in dummy mode and successfully completed then:
 		 * - no need to checksum the media afterward (done later)
 		 * - no eject to have automatic real burning */
@@ -2185,9 +2331,6 @@ brasero_burn_record_session (BraseroBurn *burn,
 
 	track = tracks->data;
 	type = brasero_track_get_checksum_type (track);
-	if (type == BRASERO_CHECKSUM_NONE)
-		return BRASERO_BURN_OK;
-
 	if (type == BRASERO_CHECKSUM_MD5
 	||  type == BRASERO_CHECKSUM_SHA1
 	||  type == BRASERO_CHECKSUM_SHA256)
@@ -2198,6 +2341,8 @@ brasero_burn_record_session (BraseroBurn *burn,
 		checksum = BRASERO_SHA1_FILE;
 	else if (type == BRASERO_CHECKSUM_SHA256_FILE)
 		checksum = BRASERO_SHA256_FILE;
+	else
+		return BRASERO_BURN_OK;
 
 	/* the idea is to push a new track on the stack with
 	 * the current disc burnt and the checksum generated
@@ -2434,13 +2579,6 @@ brasero_burn_same_src_dest_image (BraseroBurn *self,
 							     &ret_error);
 	}
 
-	/* some, like cdrdao, can't overwrite the files */
-	if (image)
-		g_remove (image);
-
-	if (toc)
-		g_remove (toc);
-
 	result = brasero_burn_session_set_image_output_full (priv->session,
 							     brasero_track_type_get_image_format (output),
 							     image,
@@ -2525,11 +2663,9 @@ again:
 	if (result != BRASERO_BURN_OK)
 		return result;
 
-	/* update the flags now before locking it since in lock function
-	 * we check the adequacy of the medium inserted. */
-	result = brasero_burn_check_session_consistency (burn, error);
-	if (result == BRASERO_BURN_CANCEL)
-		return result;
+	/* Note: we don't need to update the flags anymore
+	 * as they are updated in brasero_burn_run_tasks ()
+	 * anyway. */
 
 	if (result != BRASERO_BURN_OK) {
 		/* Tell the user his/her disc is not supported and reload */
