@@ -42,6 +42,8 @@
 
 #include "brasero-units.h"
 
+#include "brasero-track-data-cfg.h"
+
 #include "brasero-enums.h"
 #include "brasero-session.h"
 #include "brasero-status-dialog.h"
@@ -108,8 +110,20 @@ brasero_status_dialog_update (BraseroStatusDialog *self,
 	type = brasero_track_type_new ();
 	brasero_burn_session_get_input_type (priv->session, type);
 
-	if (brasero_track_type_get_has_stream (type))
-		size_str = brasero_units_get_time_string (session_bytes, TRUE, FALSE);
+	if (brasero_track_type_get_has_stream (type)) {
+		if (BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type))) {
+			guint64 free_time;
+
+			/* This is an embarassing problem: this is an approximation based on the fact that
+			 * 2 hours = 4.3GiB */
+			free_time = session_bytes * 72000LL / 47LL;
+			size_str = brasero_units_get_time_string (free_time,
+			                                          TRUE,
+			                                          TRUE);
+		}
+		else
+			size_str = brasero_units_get_time_string (session_bytes, TRUE, FALSE);
+	}
 	/* NOTE: this is perfectly fine as brasero_track_type_get_medium_type ()
 	 * will return BRASERO_MEDIUM_NONE if this is not a MEDIUM track type */
 	else if (brasero_track_type_get_medium_type (type) & BRASERO_MEDIUM_HAS_AUDIO)
@@ -119,7 +133,7 @@ brasero_status_dialog_update (BraseroStatusDialog *self,
 
 	brasero_track_type_free (type);
 
-	string = g_strdup_printf (_("Project estimated size: %s"), size_str);
+	string = g_strdup_printf (_("Estimated size: %s"), size_str);
 	g_free (size_str);
 
 	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (priv->progress), string);
@@ -150,11 +164,125 @@ brasero_status_dialog_wait_for_ready_state (BraseroStatusDialog *dialog)
 	return TRUE;
 }
 
+static gboolean
+brasero_status_dialog_deep_directory_cb (BraseroTrackDataCfg *project,
+					 const gchar *name,
+					 BraseroStatusDialog *dialog)
+{
+	gint answer;
+	gchar *string;
+	GtkWidget *message;
+
+	gtk_widget_hide (GTK_WIDGET (dialog));
+
+	string = g_strdup_printf (_("Do you really want to add \"%s\" to the selection?"), name);
+	message = gtk_message_dialog_new (GTK_WINDOW (dialog),
+					  GTK_DIALOG_DESTROY_WITH_PARENT|
+					  GTK_DIALOG_MODAL,
+					  GTK_MESSAGE_WARNING,
+					  GTK_BUTTONS_NONE,
+					  string);
+	g_free (string);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
+						  _("The children of this directory will have 7 parent directories."
+						    "\nBrasero can create an image of such a file hierarchy and burn it; but the disc may not be readable on all operating systems."
+						    "\nNOTE: Such a file hierarchy is known to work on linux."));
+
+	gtk_dialog_add_button (GTK_DIALOG (message), GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
+	gtk_dialog_add_button (GTK_DIALOG (message), _("_Add File"), GTK_RESPONSE_YES);
+
+	answer = gtk_dialog_run (GTK_DIALOG (message));
+	gtk_widget_destroy (message);
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+
+	return (answer != GTK_RESPONSE_YES);
+}
+
+static gboolean
+brasero_status_dialog_2G_file_cb (BraseroTrackDataCfg *track,
+				  const gchar *name,
+				  BraseroStatusDialog *dialog)
+{
+	gint answer;
+	gchar *string;
+	GtkWidget *message;
+
+	gtk_widget_hide (GTK_WIDGET (dialog));
+
+	string = g_strdup_printf (_("Do you really want to add \"%s\" to the selection and use the third version of ISO9660 standard to support it?"), name);
+	message = gtk_message_dialog_new (GTK_WINDOW (dialog),
+					  GTK_DIALOG_DESTROY_WITH_PARENT|
+					  GTK_DIALOG_MODAL,
+					  GTK_MESSAGE_WARNING,
+					  GTK_BUTTONS_NONE,
+					  string);
+	g_free (string);
+
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
+						  _("The size of the file is over 2 GiB. Files larger than 2 GiB are not supported by ISO9660 standard in its first and second versions (the most widespread ones)."
+						    "\nIt is recommended to use the third version of ISO9660 standard which is supported by most of the operating systems including Linux and all versions of Windows ©."
+						    "\nHowever MacOS X cannot read images created with version 3 of ISO9660 standard."));
+
+	gtk_dialog_add_button (GTK_DIALOG (message), GTK_STOCK_CANCEL, GTK_RESPONSE_NO);
+	gtk_dialog_add_button (GTK_DIALOG (message), _("_Add File"), GTK_RESPONSE_YES);
+
+	answer = gtk_dialog_run (GTK_DIALOG (message));
+	gtk_widget_destroy (message);
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+
+	return (answer != GTK_RESPONSE_YES);
+}
+
+static void
+brasero_status_dialog_joliet_rename_cb (BraseroTrackData *track,
+					BraseroStatusDialog *dialog)
+{
+	GtkResponseType answer;
+	GtkWidget *message;
+	gchar *secondary;
+
+	gtk_widget_hide (GTK_WIDGET (dialog));
+
+	message = gtk_message_dialog_new (GTK_WINDOW (dialog),
+					  GTK_DIALOG_DESTROY_WITH_PARENT|
+					  GTK_DIALOG_MODAL,
+					  GTK_MESSAGE_WARNING,
+					  GTK_BUTTONS_NONE,
+					  _("Should files be renamed to be fully Windows-compatible?"));
+
+	secondary = g_strdup_printf ("%s\n%s",
+				     _("Some files don't have a suitable name for a fully Windows-compatible CD."),
+				     _("Those names should be changed and truncated to 64 characters."));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message), "%s", secondary);
+	g_free (secondary);
+
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Disable Full Windows Compatibility"),
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Rename for Full Windows Compatibility"),
+			       GTK_RESPONSE_YES);
+
+	answer = gtk_dialog_run (GTK_DIALOG (message));
+	gtk_widget_destroy (message);
+
+	if (answer != GTK_RESPONSE_YES)
+		brasero_track_data_rm_fs (track, BRASERO_IMAGE_FS_JOLIET);
+	else
+		brasero_track_data_add_fs (track, BRASERO_IMAGE_FS_JOLIET);
+
+	gtk_widget_show (GTK_WIDGET (dialog));
+}
+
 static void
 brasero_status_dialog_wait_for_session (BraseroStatusDialog *dialog)
 {
 	BraseroStatus *status;
 	BraseroBurnResult result;
+	BraseroTrackType *track_type;
 	BraseroStatusDialogPrivate *priv;
 
 	priv = BRASERO_STATUS_DIALOG_PRIVATE (dialog);
@@ -169,6 +297,32 @@ brasero_status_dialog_wait_for_session (BraseroStatusDialog *dialog)
 
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 
+	track_type = brasero_track_type_new ();
+	brasero_burn_session_get_input_type (priv->session, track_type);
+	if (brasero_track_type_get_has_data (track_type)) {
+		GSList *tracks;
+		BraseroTrack *track;
+
+		tracks = brasero_burn_session_get_tracks (priv->session);
+		track = tracks->data;
+
+		if (BRASERO_IS_TRACK_DATA_CFG (track)) {
+			g_signal_connect (track,
+					  "joliet-rename",
+					  G_CALLBACK (brasero_status_dialog_joliet_rename_cb),
+					  dialog);
+			g_signal_connect (track,
+					  "2G-file",
+					  G_CALLBACK (brasero_status_dialog_2G_file_cb),
+					  dialog);
+			g_signal_connect (track,
+					  "deep-directory",
+					  G_CALLBACK (brasero_status_dialog_deep_directory_cb),
+					  dialog);
+		}
+	}
+	brasero_track_type_free (track_type);
+
 	brasero_status_dialog_update (dialog, status);
 	brasero_status_free (status);
 	priv->id = g_timeout_add (200,
@@ -314,9 +468,9 @@ brasero_status_dialog_new (BraseroBurnSession *session,
 			     "session", session,
 			     "transient-for", parent,
 			     "modal", TRUE,
-			     "title",  _("Project Size Estimation"),
+			     "title",  _("Size Estimation"),
 			     "message-type", GTK_MESSAGE_OTHER,
-			     "text", _("Please wait until the estimation of the project size is completed."),
-			     "secondary-text", _("All files from the project need to be analysed to complete this operation."),
+			     "text", _("Please wait until the estimation of the size is completed."),
+			     "secondary-text", _("All files need to be analysed to complete this operation."),
 			     NULL);
 }
