@@ -38,6 +38,7 @@
 #include <glib/gi18n-lib.h>
 
 #include "brasero-media.h"
+#include "brasero-media-private.h"
 
 static gboolean debug = 0;
 
@@ -50,7 +51,8 @@ static gboolean debug = 0;
 static const GOptionEntry options [] = {
 	{ "brasero-media-debug", 0, 0, G_OPTION_ARG_NONE, &debug,
 	  N_("Display debug statements on stdout for Brasero media library"),
-	  NULL }
+	  NULL },
+	{ NULL }
 };
 
 static GSList *
@@ -74,9 +76,12 @@ brasero_media_new_status (GSList *retval,
 						    BRASERO_MEDIUM_BLANK);
 
 		/* NOTE about BR-R they can be "formatted" but they are never
-		 * unformatted since by default they'll be used as sequential */
+		 * unformatted since by default they'll be used as sequential.
+		 * We don't consider DVD-RW as unformatted as in this case
+		 * they are treated as DVD-RW in sequential mode and therefore
+		 * don't require any formatting. */
 		if (!(media & BRASERO_MEDIUM_RAM)
-		&&   (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_REWRITABLE)
+		&&   (BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_DVDRW_PLUS)
 		||    BRASERO_MEDIUM_IS (media, BRASERO_MEDIUM_BD|BRASERO_MEDIUM_REWRITABLE))) {
 			if (type & BRASERO_MEDIUM_UNFORMATTED)
 				retval = brasero_media_add_to_list (retval,
@@ -519,6 +524,25 @@ brasero_media_library_start (void)
 		return;
 	}
 
+	BRASERO_MEDIA_LOG ("Initializing Brasero-media %i.%i.%i",
+	                    BRASERO_MAJOR_VERSION,
+	                    BRASERO_MINOR_VERSION,
+	                    BRASERO_SUB);
+
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	/* Work around: because on OpenSolaris brasero posiblely be run
+	 * as root for a user with 'Primary Administrator' profile,
+	 * a root dbus session will be autospawned at that time.
+	 * This fix is to work around
+	 * http://bugzilla.gnome.org/show_bug.cgi?id=526454
+	 */
+	g_setenv ("DBUS_SESSION_BUS_ADDRESS", "autolaunch:", TRUE);
+#endif
+
+	/* Initialize external libraries (threads... */
+	if (!g_thread_supported ())
+		g_thread_init (NULL);
+
 	/* Initialize i18n */
 	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
