@@ -508,7 +508,12 @@ brasero_libburn_start_record (BraseroLibburn *self,
 	 * checking thing earlier ourselves. Now there is a proper
 	 * disc and tray is locked. */
 	opts = burn_write_opts_new (priv->ctx->drive);
-	burn_write_opts_set_perform_opc (opts, 0);
+
+	/* only turn this on for CDs */
+	if ((media & BRASERO_MEDIUM_CD) != 0)
+		burn_write_opts_set_perform_opc (opts, 0);
+	else
+		burn_write_opts_set_perform_opc (opts, 0);
 
 	if (flags & BRASERO_BURN_FLAG_DAO)
 		burn_write_opts_set_write_type (opts,
@@ -533,16 +538,22 @@ brasero_libburn_start_record (BraseroLibburn *self,
 		}
 	}
 
-	if (!BRASERO_MEDIUM_RANDOM_WRITABLE (media))
+	if (!BRASERO_MEDIUM_RANDOM_WRITABLE (media)) {
+		BRASERO_JOB_LOG (BRASERO_JOB (self), "Setting multi %i", (flags & BRASERO_BURN_FLAG_MULTI) != 0);
 		burn_write_opts_set_multi (opts, (flags & BRASERO_BURN_FLAG_MULTI) != 0);
+	}
 
 	burn_write_opts_set_underrun_proof (opts, (flags & BRASERO_BURN_FLAG_BURNPROOF) != 0);
+	BRASERO_JOB_LOG (BRASERO_JOB (self), "Setting burnproof %i", (flags & BRASERO_BURN_FLAG_BURNPROOF) != 0);
+
 	burn_write_opts_set_simulate (opts, (flags & BRASERO_BURN_FLAG_DUMMY) != 0);
+	BRASERO_JOB_LOG (BRASERO_JOB (self), "Setting dummy %i", (flags & BRASERO_BURN_FLAG_DUMMY) != 0);
 
 	brasero_job_get_rate (BRASERO_JOB (self), &rate);
 	burn_drive_set_speed (priv->ctx->drive, rate, 0);
 
 	if (burn_precheck_write (opts, priv->ctx->disc, reason, 0) < 1) {
+		BRASERO_JOB_LOG (BRASERO_JOB (self), "Precheck failed %s", reason);
 		g_set_error (error,
 			     BRASERO_BURN_ERROR,
 			     BRASERO_BURN_ERROR_GENERAL,
@@ -646,7 +657,8 @@ brasero_libburn_start_erase (BraseroLibburn *self,
 		BRASERO_JOB_NOT_SUPPORTED (self);
 
 	/* This is the "fast option": basically we only write 64 Kib of 0 from
-	 * /dev/null */
+	 * /dev/null. If we reached that part it means we're dealing with
+	 * overwrite media. */
 	fd = open ("/dev/null", O_RDONLY);
 	if (fd == -1) {
 		int errnum = errno;
@@ -724,9 +736,25 @@ brasero_libburn_start (BraseroJob *job,
 
 	brasero_job_get_action (job, &action);
 	if (action == BRASERO_JOB_ACTION_RECORD) {
-		priv->ctx = brasero_libburn_common_ctx_new (job, error);
-		if (!priv->ctx)
+		GError *ret_error = NULL;
+
+		/* TRUE is a context that helps to adapt action
+		 * messages like for DVD+RW which need a
+		 * pre-formatting before actually writing
+		 * and without this we would not know if
+		 * we are actually formatting or just pre-
+		 * formatting == starting to record */
+		priv->ctx = brasero_libburn_common_ctx_new (job, TRUE, &ret_error);
+		if (!priv->ctx) {
+			if (ret_error && ret_error->code == BRASERO_BURN_ERROR_DRIVE_BUSY) {
+				g_propagate_error (error, ret_error);
+				return BRASERO_BURN_RETRY;
+			}
+
+			if (error)
+				g_propagate_error (error, ret_error);
 			return BRASERO_BURN_ERR;
+		}
 
 		result = brasero_libburn_start_record (self, error);
 		if (result != BRASERO_BURN_OK)
@@ -738,9 +766,19 @@ brasero_libburn_start (BraseroJob *job,
 						FALSE);
 	}
 	else if (action == BRASERO_JOB_ACTION_ERASE) {
-		priv->ctx = brasero_libburn_common_ctx_new (job, error);
-		if (!priv->ctx)
+		GError *ret_error = NULL;
+
+		priv->ctx = brasero_libburn_common_ctx_new (job, FALSE, &ret_error);
+		if (!priv->ctx) {
+			if (ret_error && ret_error->code == BRASERO_BURN_ERROR_DRIVE_BUSY) {
+				g_propagate_error (error, ret_error);
+				return BRASERO_BURN_RETRY;
+			}
+
+			if (error)
+				g_propagate_error (error, ret_error);
 			return BRASERO_BURN_ERR;
+		}
 
 		result = brasero_libburn_start_erase (self, error);
 		if (result != BRASERO_BURN_OK)
@@ -932,8 +970,20 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 	/* libburn has no OVERBURN capabilities */
 
 	/* CD(R)W */
-	BRASERO_PLUGIN_ADD_STANDARD_CDR_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
-	BRASERO_PLUGIN_ADD_STANDARD_CDRW_FLAGS (plugin, BRASERO_BURN_FLAG_OVERBURN);
+	/* Use DAO for first session since AUDIO need it to write CD-TEXT
+	 * Though libburn is unable to write CD-TEXT.... */
+	/* Note: when burning multiple tracks to a CD (like audio for example)
+	 * in dummy mode with TAO libburn will fail probably because it does
+	 * not use a correct next writable address for the second track (it uses
+	 * the same as for track #1). So remove dummy mode.
+	 * This is probably the same reason why it fails at merging another
+	 * session to a data CD in dummy mode. */
+	BRASERO_PLUGIN_ADD_STANDARD_CDR_FLAGS (plugin,
+	                                       BRASERO_BURN_FLAG_OVERBURN|
+	                                       BRASERO_BURN_FLAG_DUMMY);
+	BRASERO_PLUGIN_ADD_STANDARD_CDRW_FLAGS (plugin,
+	                                        BRASERO_BURN_FLAG_OVERBURN|
+	                                        BRASERO_BURN_FLAG_DUMMY);
 
 	/* audio support for CDs only */
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
