@@ -494,8 +494,10 @@ brasero_app_dialog (BraseroApp *app,
 
 	priv = BRASERO_APP_PRIVATE (app);
 
-	if (priv->mainwin)
+	if (priv->mainwin) {
 		toplevel = GTK_WINDOW (priv->mainwin);
+		gtk_widget_show (priv->mainwin);
+	}
 	else if (!priv->toplevel) {
 		is_on_top = TRUE;
 		toplevel = NULL;
@@ -955,9 +957,6 @@ brasero_app_set_toplevel (BraseroApp *app, GtkWindow *window)
 		}
 	}
 	else {
-		gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (priv->mainwin));
-		gtk_window_set_modal (GTK_WINDOW (window), TRUE);
-
 		/* hide main dialog if it is shown */
 		gtk_widget_hide (GTK_WIDGET (priv->mainwin));
 
@@ -1127,6 +1126,192 @@ on_configure_event_cb (GtkWidget *widget,
 	return FALSE;
 }
 
+gboolean
+brasero_app_open_project (BraseroApp *app,
+                          const gchar *uri,
+                          gboolean is_playlist,
+                          gboolean warn_user,
+                          gboolean burn)
+{
+	BraseroSessionCfg *session;
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	session = brasero_session_cfg_new ();
+
+#ifdef BUILD_PLAYLIST
+
+	if (is_playlist) {
+		if (!brasero_project_open_audio_playlist_project (uri, BRASERO_BURN_SESSION (session), warn_user))
+			return FALSE;
+	}
+	else
+
+#endif
+	
+	if (!brasero_project_open_project_xml (uri, BRASERO_BURN_SESSION (session), warn_user))
+		return FALSE;
+
+	if (!priv->projects) {
+		brasero_app_create_mainwin (app);
+		brasero_project_manager_open_session (BRASERO_PROJECT_MANAGER (priv->projects), session, burn);
+		brasero_app_run_mainwin (app);
+	}
+	else
+		brasero_project_manager_open_session (BRASERO_PROJECT_MANAGER (priv->projects), session, burn);
+
+	return TRUE;
+}
+
+static gboolean
+brasero_app_open_by_mime (BraseroApp *app,
+                          const gchar *uri,
+                          const gchar *mime,
+                          gboolean warn_user)
+{
+	BraseroAppPrivate *priv;
+
+	priv = BRASERO_APP_PRIVATE (app);
+
+	if (!mime) {
+		/* that can happen when the URI could not be identified */
+		return FALSE;
+	}
+
+	/* When our files/description of x-brasero mime type is not properly 
+	 * installed, it's returned as application/xml, so check that too. */
+	if (!strcmp (mime, "application/x-brasero")
+	||  !strcmp (mime, "application/xml"))
+		return brasero_app_open_project (app, uri, FALSE, warn_user, FALSE);
+
+#ifdef BUILD_PLAYLIST
+
+	else if (!strcmp (mime, "audio/x-scpls")
+	     ||  !strcmp (mime, "audio/x-ms-asx")
+	     ||  !strcmp (mime, "audio/x-mp3-playlist")
+	     ||  !strcmp (mime, "audio/x-mpegurl"))
+		return brasero_app_open_project (app, uri, TRUE,  warn_user, FALSE);
+
+#endif
+
+	else if (!strcmp (mime, "application/x-cd-image")
+	     ||  !strcmp (mime, "application/x-cdrdao-toc")
+	     ||  !strcmp (mime, "application/x-toc")
+	     ||  !strcmp (mime, "application/x-cue")) {
+		if (priv->projects)
+			brasero_project_manager_iso (BRASERO_PROJECT_MANAGER (priv->projects), uri);
+		else
+			brasero_app_burn_image (app, uri);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+brasero_app_open_uri (BraseroApp *app,
+                      const gchar *uri_arg,
+                      gboolean warn_user)
+{
+	gchar *uri;
+	GFile *file;
+	GFileInfo *info;
+	BraseroProjectType type;
+
+	/* FIXME: make that asynchronous */
+	/* NOTE: don't follow symlink because we want to identify them */
+	file = g_file_new_for_commandline_arg (uri_arg);
+	if (!file)
+		return BRASERO_PROJECT_TYPE_INVALID;
+
+	info = g_file_query_info (file,
+				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+				  G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
+				  G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
+				  G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+				  NULL,
+				  NULL);
+
+	if (!info) {
+		g_object_unref (file);
+		return BRASERO_PROJECT_TYPE_INVALID;
+	}
+
+	/* if that's a symlink, redo it on its target to get the real mime type
+	 * that usually also depends on the extension of the target:
+	 * ex: an iso file with the extension .iso will be seen as octet-stream
+	 * if the symlink hasn't got any extention at all */
+	while (g_file_info_get_is_symlink (info)) {
+		const gchar *target;
+		GFileInfo *tmp_info;
+		GFile *tmp_file;
+		GError *error = NULL;
+
+		target = g_file_info_get_symlink_target (info);
+		if (!g_path_is_absolute (target)) {
+			gchar *parent;
+			gchar *tmp;
+
+			tmp = g_file_get_path (file);
+			parent = g_path_get_dirname (tmp);
+			g_free (tmp);
+
+			target = g_build_filename (parent, target, NULL);
+			g_free (parent);
+		}
+
+		tmp_file = g_file_new_for_commandline_arg (target);
+		tmp_info = g_file_query_info (tmp_file,
+					      G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+					      G_FILE_ATTRIBUTE_STANDARD_IS_SYMLINK ","
+					      G_FILE_ATTRIBUTE_STANDARD_SYMLINK_TARGET,
+					      G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+					      NULL,
+					      &error);
+		if (!tmp_info) {
+			g_object_unref (tmp_file);
+			break;
+		}
+
+		g_object_unref (info);
+		g_object_unref (file);
+
+		info = tmp_info;
+		file = tmp_file;
+	}
+
+	uri = g_file_get_uri (file);
+	if (g_file_query_exists (file, NULL)
+	&& g_file_info_get_content_type (info)) {
+		const gchar *mime;
+
+		mime = g_file_info_get_content_type (info);
+	  	type = brasero_app_open_by_mime (app, uri, mime, warn_user);
+        } 
+	else if (warn_user) {
+		gchar *string;
+
+		string = g_strdup_printf (_("The project \"%s\" does not exist"), uri);
+		brasero_app_alert (app,
+				   _("Error while loading the project"),
+				   string,
+				   GTK_MESSAGE_ERROR);
+		g_free (string);
+
+		type = BRASERO_PROJECT_TYPE_INVALID;
+	}
+	else
+		type = BRASERO_PROJECT_TYPE_INVALID;
+
+	g_free (uri);
+	g_object_unref (file);
+	g_object_unref (info);
+
+	return type;
+}
+
 static void
 brasero_app_recent_open (GtkRecentChooser *chooser,
 			 BraseroApp *app)
@@ -1166,9 +1351,10 @@ brasero_app_recent_open (GtkRecentChooser *chooser,
 	}
 
 	/* Make sure it is no longer one shot */
-	brasero_project_manager_open_by_mime (BRASERO_PROJECT_MANAGER (priv->projects),
-					      uri,
-					      mime);
+	brasero_app_open_by_mime (app,
+	                          uri,
+	                          mime,
+	                          TRUE);
 	gtk_recent_info_unref (item);
 	g_free (uri);
 }
@@ -1327,6 +1513,9 @@ brasero_app_create_mainwin (BraseroApp *app)
 
 	priv = BRASERO_APP_PRIVATE (app);
 
+	if (priv->mainwin)
+		return;
+
 	/* New window */
 	priv->mainwin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_icon_name (GTK_WINDOW (priv->mainwin), "brasero");
