@@ -37,7 +37,6 @@
 
 #include <gconf/gconf-client.h>
 
-#include "burn-debug.h"
 #include "brasero-layout.h"
 #include "brasero-preview.h"
 #include "brasero-project.h"
@@ -82,6 +81,8 @@ struct BraseroLayoutPrivate {
 	GtkWidget *project;
 
 	GtkWidget *combo;
+	GtkWidget *top_box;
+
 	BraseroLayoutType ctx_type;
 	BraseroLayoutItem *active_item;
 
@@ -498,7 +499,6 @@ brasero_layout_set_side_pane_visible (BraseroLayout *layout,
 		/* Now tell the project which source it gets URIs from */
 		source = brasero_layout_item_get_object (layout->priv->active_item);
 		if (!BRASERO_IS_URI_CONTAINER (source)) {
-			BRASERO_BURN_LOG ("Item is not an URI container");
 			brasero_project_set_source (BRASERO_PROJECT (layout->priv->project),
 						    NULL);
 		}
@@ -752,6 +752,9 @@ brasero_layout_add_source (BraseroLayout *layout,
 			    ITEM_COL, item,
 			    VISIBLE_COL, TRUE,
 			    -1);
+
+	if (gtk_tree_model_iter_n_children (model, NULL) > 1)
+		gtk_widget_show (layout->priv->top_box);
 }
 
 /**************************** empty view callback ******************************/
@@ -791,7 +794,6 @@ brasero_layout_combo_changed_cb (GtkComboBox *combo,
 	brasero_layout_object_set_context (source, layout->priv->ctx_type);
 
 	if (!BRASERO_IS_URI_CONTAINER (source)) {
-		BRASERO_BURN_LOG ("Item is not an URI container");
 		brasero_project_set_source (BRASERO_PROJECT (layout->priv->project), NULL);
 	}
 	else if (!is_visible) {
@@ -1053,13 +1055,13 @@ brasero_layout_change_type (BraseroLayout *layout,
 		switch (layout_type) {
 			case BRASERO_LAYOUT_TOP:
 			case BRASERO_LAYOUT_LEFT:
-				gtk_paned_pack2 (GTK_PANED (layout->priv->pane), source_pane, TRUE, FALSE);
+				gtk_paned_pack2 (GTK_PANED (layout->priv->pane), source_pane, TRUE, TRUE);
 				gtk_paned_pack1 (GTK_PANED (layout->priv->pane), project_pane, TRUE, FALSE);
 				break;
 
 			case BRASERO_LAYOUT_BOTTOM:
 			case BRASERO_LAYOUT_RIGHT:
-				gtk_paned_pack1 (GTK_PANED (layout->priv->pane), source_pane, TRUE, FALSE);
+				gtk_paned_pack1 (GTK_PANED (layout->priv->pane), source_pane, TRUE, TRUE);
 				gtk_paned_pack2 (GTK_PANED (layout->priv->pane), project_pane, TRUE, FALSE);
 				break;
 
@@ -1412,9 +1414,9 @@ brasero_layout_init (BraseroLayout *obj)
 
 	if (obj->priv->layout_type == BRASERO_LAYOUT_TOP
 	||  obj->priv->layout_type == BRASERO_LAYOUT_LEFT)
-		gtk_paned_pack2 (GTK_PANED (obj->priv->pane), alignment, TRUE, FALSE);
+		gtk_paned_pack2 (GTK_PANED (obj->priv->pane), alignment, TRUE, TRUE);
 	else
-		gtk_paned_pack1 (GTK_PANED (obj->priv->pane), alignment, TRUE, FALSE);
+		gtk_paned_pack1 (GTK_PANED (obj->priv->pane), alignment, TRUE, TRUE);
 
 	obj->priv->main_box = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment), obj->priv->main_box);
@@ -1427,9 +1429,9 @@ brasero_layout_init (BraseroLayout *obj)
 							      NULL,
 							      NULL);
 
-	/* close button and  combo */
+	/* close button and combo. Don't show it now. */
 	box = gtk_hbox_new (FALSE, 6);
-	gtk_widget_show (box);
+	obj->priv->top_box = box;
 	gtk_box_pack_start (GTK_BOX (obj->priv->main_box),
 			    box,
 			    FALSE,
