@@ -773,8 +773,7 @@ brasero_disc_get_use_info_notebook (BraseroProject *project)
 }
 
 /********************************** size ***************************************/
-
-gchar *
+static gchar *
 brasero_project_get_sectors_string (gint64 sectors,
 				    BraseroTrackType *type)
 {
@@ -1203,6 +1202,11 @@ brasero_project_finalize (GObject *object)
 	BraseroProject *cobj;
 	cobj = BRASERO_PROJECT(object);
 
+	if (cobj->priv->session) {
+		g_object_unref (cobj->priv->session);
+		cobj->priv->session = NULL;
+	}
+
 	if (cobj->priv->project)
 		g_free (cobj->priv->project);
 
@@ -1378,8 +1382,6 @@ brasero_project_drive_properties (BraseroProject *project)
 					      NULL);
 	g_free (header);
 
-	brasero_app_set_toplevel (brasero_app_get_default (), GTK_WINDOW (dialog));
-
 	button = brasero_utils_make_button (_("_Burn"),
 					    NULL,
 					    "media-optical-burn",
@@ -1411,8 +1413,10 @@ brasero_project_drive_properties (BraseroProject *project)
 	brasero_track_type_free (track_type);
 
 	medium_prop = brasero_drive_properties_new (project->priv->session);
-	gtk_widget_show (medium_prop);
 	gtk_box_pack_start (GTK_BOX (box), medium_prop, TRUE, TRUE, 0);
+	gtk_widget_show (medium_prop);
+
+	brasero_app_set_toplevel (brasero_app_get_default (), GTK_WINDOW (dialog));
 
 	/* launch the dialog */
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1472,7 +1476,6 @@ brasero_project_image_properties (BraseroProject *project)
 	brasero_track_type_free (track_type);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-	gtk_widget_show (dialog);
 
 	/* launch the dialog */
 	answer = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -2438,6 +2441,7 @@ brasero_project_save_project_real (BraseroProject *project,
 		return FALSE;
 	}
 
+	brasero_project_setup_session (project, BRASERO_BURN_SESSION (project->priv->session));
         type = brasero_project_get_session_type (project);
 
 	if (save_type == BRASERO_PROJECT_SAVE_XML
@@ -2671,6 +2675,7 @@ brasero_project_save_session (BraseroProject *project,
 		return FALSE;
 	}
 
+	brasero_project_setup_session (project, BRASERO_BURN_SESSION (project->priv->session));
 	if (!brasero_project_save_project_xml (BRASERO_BURN_SESSION (project->priv->session), uri)) {
 		GtkResponseType response;
 		GtkWidget *dialog;
