@@ -35,7 +35,7 @@
 
 #include <glib.h>
 #include <glib-object.h>
-#include <gtk/gtkhbox.h>
+#include <gtk/gtk.h>
 
 #include <beagle/beagle-query.h>
 
@@ -52,12 +52,12 @@ G_BEGIN_DECLS
 typedef struct BraseroSearchEntryPrivate BraseroSearchEntryPrivate;
 
 typedef struct {
-	GtkHBox parent;
+	GtkTable parent;
 	BraseroSearchEntryPrivate *priv;
 } BraseroSearchEntry;
 
 typedef struct {
-	GtkHBoxClass parent_class;
+	GtkTableClass parent_class;
 
 	void	(*activated)	(BraseroSearchEntry *entry);
 
