@@ -30,9 +30,7 @@
 #include <glib.h>
 #include <glib-object.h>
 
-#include <gtk/gtkdialog.h>
-
-#include "burn-basics.h"
+#include <gtk/gtk.h>
 
 G_BEGIN_DECLS
 
@@ -77,4 +75,5 @@ brasero_song_props_set_properties (BraseroSongProps *self,
 				   gint64 start,
 				   gint64 end,
 				   gint64 gap);
+
 #endif /* SONG_PROPERTIES_H */
